import os
import sys
from pathlib import Path
from Crypto.Cipher import AES

# Ścieżki plików
in_path    = Path("postprocessed.bin")       # plik z danymi po LFSR
key_path   = Path("aes_key.bin")
nonce_path = Path("aes_nonce.bin")
out_path   = Path("aes.bin")

# 1. Wczytaj dane po LFSR
if not in_path.exists():
    raise FileNotFoundError(f"Plik źródłowy {in_path} nie istnieje.")
data = in_path.read_bytes()
length = len(data)
print(f"[AES] Wczytano {length:,} bajtów danych po LFSR")

# 2. Wczytaj lub wygeneruj klucz i nonce
if key_path.exists() and nonce_path.exists():
    key   = key_path.read_bytes()
    nonce = nonce_path.read_bytes()
    print("[AES] Wczytano istniejący klucz i nonce")
else:
    key   = os.urandom(32)  # 256-bitowy klucz
    nonce = os.urandom(8)   # 64-bitowy nonce
    key_path.write_bytes(key)
    nonce_path.write_bytes(nonce)
    print("[AES] Wygenerowano nowy klucz i nonce")

# 3. Inicjalizacja AES-256 CTR i generacja keystream
cipher   = AES.new(key, AES.MODE_CTR, nonce=nonce)
keystream = cipher.encrypt(bytes(length))

# 4. Whitening (XOR keystream z danymi) z paskiem postępu
def print_progress_bar(current, total, bar_length=40):
    percent = current / total
    arrow = "#" * int(percent * bar_length)
    spaces = "-" * (bar_length - len(arrow))
    sys.stdout.write(f"\r[AES] Whitening: |{arrow}{spaces}| {percent * 100:5.1f}% ({current:,}/{total:,})")
    sys.stdout.flush()

whitened = bytearray()
for idx, (byte, ks) in enumerate(zip(data, keystream), start=1):
    whitened.append(byte ^ ks)
    if idx % 100_000 == 0 or idx == length:
        print_progress_bar(idx, length)

print()  # Nowa linia po zakończeniu paska

# 5. Zapis wyników
out_path.write_bytes(whitened)
print(f"[AES] Zapisano wyjście: {out_path}")
