import numpy as np
import matplotlib.pyplot as plt

# Ścieżka do pliku binarnego
file_path = 'btc_returns_13MB_8bit.bin'

# Wczytywanie bajtów jako uint8
with open(file_path, 'rb') as f:
    data = f.read()
bytes_array = np.frombuffer(data, dtype=np.uint8)

# Obliczanie rozkładu i entropii
counts = np.bincount(bytes_array, minlength=256)
probs = counts / counts.sum()
nonzero_probs = probs[probs > 0]
entropy = -np.sum(nonzero_probs * np.log2(nonzero_probs))
print(f"Entropia rozkładu: {entropy:.3f} bitów")

# Rysowanie histogramu z prawdopodobieństwem
plt.figure(figsize=(10,6))
plt.hist(bytes_array, bins=256, range=(0,255), edgecolor='black', density=True)
plt.title("Histogram rozkładu 8-bitowych liczb losowych")
plt.xlabel("Wartość bajtu (0–255)")
plt.ylabel("Prawdopodobieństwo")
plt.grid(True)
plt.tight_layout()
plt.show()
